# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""The HierarchichalVeto algorithm
"""

from ._version import get_versions

__version__ = get_versions()['version']
__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
__credits__ = 'Joshua Smith <joshua.smith@ligo.org>'

del get_versions



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for `hveto.plot`
"""

import pytest
import tempfile

from numpy import random

from matplotlib import use
use('agg')

from .. import plot  # noqa: E402


@pytest.mark.parametrize('num', (10, 50, 200))
def test_drop_plot(num):
    random.seed(0)
    # this test just makes sure the drop plot code runs end-to-end
    channels = ['X1:TEST-%d' % i for i in range(num)]
    old = dict(zip(channels, random.normal(size=num)))
    new = dict(zip(channels, random.normal(size=num)))
    with tempfile.NamedTemporaryFile(suffix='.png') as png:
        plot.significance_drop(png.name, old, new)

    with tempfile.NamedTemporaryFile(suffix='.svg') as svg:
        plot.significance_drop(svg.name, old, new)



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Plotting routines for hveto
"""

import warnings
from math import (log10, floor)
from io import BytesIO

import numpy

from lxml import etree

from matplotlib import rcParams
from matplotlib.colors import LogNorm

from gwpy.plot import Plot

from gwdetchar.plot import texify

__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
__credits__ = 'Josh Smith, Joe Areeda, Alex Urban'

rcParams.update({
    # custom Hveto formatting
    'figure.subplot.bottom': 0.17,
    'figure.subplot.left': 0.12,
    'figure.subplot.right': 0.88,
    'figure.subplot.top': 0.90,
    'xtick.labelsize': 16,
    'ytick.labelsize': 16,
    'axes.labelsize': 24,
    'axes.labelpad': 2,
    'axes.titlesize': 24,
    'grid.color': 'gray',
    'grid.alpha': 0.5,
})

SHOW_HIDE_JAVASCRIPT = """
    <script type="text/ecmascript">
    <![CDATA[

    function init(evt) {
        if ( window.svgDocument == null ) {
            svgDocument = evt.target.ownerDocument;
            }
        }

    function ShowTooltip(obj) {
        var cur = obj.id.substr(obj.id.lastIndexOf('-')+1);

        var tip = svgDocument.getElementById('tooltip-' + cur);
        tip.setAttribute('visibility',"visible")
        }

    function HideTooltip(obj) {
        var cur = obj.id.substr(obj.id.lastIndexOf('-')+1);
        var tip = svgDocument.getElementById('tooltip-' + cur);
        tip.setAttribute('visibility',"hidden")
        }

    ]]>
    </script>"""


# -- set up default header plot captions

HEADER_CAPTION = {
    'HISTOGRAM':
        "Histogram of number of triggers in the primary channel before the "
        "heveto analysis, but after the data quality flag cuts (red) compared "
        "with the number after vetoes from all hveto rounds have been applied "
        "(blue) versus the signal-to-noise ratio of those triggers.",
    'ROC':
        "The fraction of the primary channel triggers vetoed (fractional "
        "efficiency) versus the fraction of livetime that is vetoed "
        "(fractional deadtime) for each hveto round (blue dots). Guidelines "
        "are given for efficiency/deadtime of 1 (the value expected for "
        "random chance) and higher.",
    'TIME':
        "Frequency versus time graph of all triggers in the primary channel "
        "before the hveto analysis (black dots) and those triggers that are "
        "vetoed by a given round (symbols).",
    'SNR_TIME':
        "Signal-to-noise ratio versus time graph of all triggers in the "
        "primary channel before the hveto analysis (black dots) and those "
        "triggers that are vetoed by a given round (symbols)."
}


# -- set up default round winner plot captions

ROUND_CAPTION = {
    'HISTOGRAM':
        "Histogram of number of triggers in the primary channel before this "
        "round of hveto (red) compared with the number after vetoes from this "
        "round have been applied (blue) versus the signal-to-noise ratio of "
        "those triggers.",
    'SNR_TIME':
        "Signal-to-noise ratio versus time graph of all triggers in the "
        "primary channel before this round of hveto (black dots) and those "
        "triggers that are vetoed by this round (red plusses).",
    'SNR':
        "Signal-to-noise ratio versus frequency graph of all triggers in the "
        "primary channel before this round of hveto (black dots) and those "
        "triggers that are vetoed by this round (red plusses).",
    'TIME':
        "Frequency versus time graph of all triggers in the primary channel "
        "before this hveto round (dots colored by signal-to-noise ratio) and "
        "those triggers that are vetoed by this round (red plusses).",
    'USED_SNR_TIME':
        "Signal-to-noise ratio versus time graph of all triggers in the "
        "auxiliary channel above the threshold selected for this round (black "
        "dots, these are the triggers used to construct the veto) and the "
        "primary channel triggers that are vetoed in this round (red "
        "plusses). This can indicate whether, for example, louder triggers in "
        "the auxiliary channel are used to veto quieter channels in the "
        "primary channel, as might be expected for an external disturbance.",
    'AUX_SNR_TIME':
        "Signal-to-noise ratio versus time graph of all triggers in the "
        "auxiliary channel before this round (black dots), those above the "
        "threshold selected for this round (yellow plusses, these are the "
        "triggers used to construct the veto), and those triggers that "
        "actually veto one of the primary channel triggers (red plusses).",
    'AUX_SNR_FREQUENCY':
        "Signal-to-noise ratio versus frequency graph of all triggers in the "
        "auxiliary channel before this round (black dots), those above the "
        "threshold selected for this round (yellow plusses, these are the "
        "triggers used to construct the veto), and those triggers that "
        "actually veto one of the primary channel triggers (red plusses).",
    'AUX_FREQUENCY_TIME':
        "Frequency versus time graph of all triggers in the auxiliary channel "
        "before this round (dots colored by signal-to-noise ratio), those "
        "above the threshold selected for this round (yellow plusses, these "
        "are the triggers used to construct the veto), and those triggers "
        "that actually veto one of the primary channel triggers (red "
        "plusses).",
    'SIG_DROP':
        "This plot includes interactive features (channel names appear when "
        "pointed to with your mouse cursor) that can be accessed by opening "
        "the plot in a new tab. The statistical significance value (based on "
        "Poisson statistics) for the best SNR and time window combination for "
        "each auxiliary channel before and after this round are shown as a "
        "baton. The round's winning channel, which had the highest "
        "significance, is shown in yellow. The top of the yellow baton is the "
        "significance of this channel before this round and the bottom of the "
        "baton is its significance in the next round, after its triggers "
        "above this round's SNR threshold and time window have been removed "
        "(note that this channel may have nonzero significance in the next "
        "round because it may still have triggers left at a lower SNR "
        "threshold). Blue batons are for channels whose significance dropped "
        "after this round (indicating that that channel had some trigger "
        "times in common with the winner) and red batons are for channels "
        "whose significance increased in the next round (due to less "
        "livetime)."
}


# -- utilities ----------------------------------------------------------------

COLUMN_LABEL = {
    'peal_frequency': r"Frequency [Hz]",
    'central_freq': r"Frequency [Hz]",
    'frequency': r"Frequency [Hz]",
    'mchirp': r"Chirp mass [M$_\odot$]",
    'new_snr': r"$\chi^2$-weighted signal-to-noise ratio (New SNR)",
    'peak_frequency': r"Frequency [Hz]",
    'rho': r"$\rho$",
    'snr': r"Signal-to-noise ratio (SNR)",
    'template_duration': r"Template duration [s]",
}


def get_column_label(column):
    try:
        return COLUMN_LABEL[column]
    except KeyError:
        return r'\texttt{{{0}}}'.format(column)


# -- Functions ----------------------------------------------------------------

def before_after_histogram(
        outfile, x, y, label1='Before', label2='After',
        bins=100, histtype='stepfilled', range=None, figsize=[9, 6], **kwargs):
    """Plot a histogram of SNR for two event distributions
    """
    # format axis arguments
    axargs = {
        'xscale': 'log',
        'xlabel': 'Loudness',
        'yscale': 'log',
        'ylabel': 'Number of events',
    }
    axargs.update(kwargs)
    # create figure
    plot = Plot(figsize=figsize)
    ax = plot.gca()
    # make histogram
    if range is None:
        range = min(map(numpy.min, (x, y))), max(map(numpy.max, (x, y)))
    axargs.setdefault('xlim', range)
    histargs = {
        'range': range,
        'histtype': histtype,
        'bins': bins,
        'linewidth': 2,
        'logbins': axargs['xscale'] == 'log',
        'alpha': .8,
    }
    ax.hist(x, label=label1, facecolor='red', edgecolor='darkred',
            **histargs)
    ax.hist(y, label=label2, facecolor='dodgerblue', edgecolor='blue',
            **histargs)
    # add legend
    ax.legend(loc='upper right')
    # format axes
    axargs.setdefault('ylim', (.5, ax.yaxis.get_data_interval()[1] * 1.05))
    _finalize_plot(plot, ax, outfile, **axargs)


def veto_scatter(outfile, a, b, label1='All', label2='Vetoed', x='time',
                 y='snr', color=None, clim=None, clabel=None, cmap=None,
                 clog=True, figsize=[9, 6], **kwargs):
    """Plot an x-y scatter of all/vetoed events
    """
    # format axis arguments
    axargs = {
        'yscale': 'log',
        'ylabel': 'Loudness',
    }
    axargs['xscale'] = 'auto-gps' if x == 'time' else 'log'
    if isinstance(y, (list, tuple)):
        ya = y[0]
        yb = y[1]
    else:
        ya = yb = y

    axargs.update(kwargs)
    # create figure
    plot = Plot(figsize=figsize)
    ax = plot.gca()
    # add data
    if color is None:
        ax.scatter(a[x], a[ya], color='black', marker='o', label=label1, s=40)
    else:
        colorargs = {'edgecolor': 'none'}
        if clim:
            colorargs['vmin'] = clim[0]
            colorargs['vmax'] = clim[1]
            if clog:
                colorargs['norm'] = LogNorm(vmin=clim[0], vmax=clim[1])
        a = a.copy()
        a.sort(color)
        m = ax.scatter(a[x], a[ya], c=a[color], label=label1, **colorargs)
        # add colorbar
        ax.colorbar(mappable=m, cmap=cmap, label=clabel)
    if isinstance(b, (list, tuple)) and len(b) == 2:
        # aux channel used/coinc (probably)
        colors = [{'color': c} for c in (
            '#ffd200',  # yellow
            '#d62728',  # red
        )]
    elif isinstance(b, (list, tuple)):
        colors = list(rcParams['axes.prop_cycle'])
    else:
        b = [b]
        label2 = [label2]
        colors = [{'color': '#d62728'}]
    for i, data in enumerate(b):
        # setting the color here looks complicated, but is just a fancy
        # way of looping through the color cycle when scattering, but using
        # red if we only have one other data set
        ax.scatter(data[x], data[yb], marker='+', linewidth=1.5,
                   label=label2[i], s=40, **colors[i % len(colors)])
    # add legend
    if ax.get_legend_handles_labels()[0]:
        legargs = {
            'loc': 'upper left',
            'bbox_to_anchor': (1.01, 1),
            'borderaxespad': 0,
            'numpoints': 1,
            'scatterpoints': 1,
            'handlelength': 1,
            'handletextpad': .5
        }
        legargs.update(dict(
            (x[7:], axargs.pop(x)) for x in list(axargs.keys()) if
            x.startswith('legend_')
        ))
        ax.legend(**legargs)
    # finalize
    for axis in ['x', 'y']:
        # get data limits
        lim = list(getattr(ax, '%saxis' % axis).get_data_interval())
        # use given ybound
        lim[0] = axargs.get('%sbound' % axis, lim[0])
        # scale out for visual
        lim[0] *= 0.95
        lim[1] *= 1.05
        # handle logs
        if axargs.get("%sscale" % axis, "linear") == "log" and lim[0] <= 0.:
            lim[0] = None
        axargs.setdefault('%slim' % axis, lim)
    _finalize_plot(plot, ax, outfile, **axargs)


def _finalize_plot(plot, ax, outfile, bbox_inches=None, close=True, **axargs):
    xlim = axargs.pop('xlim', None)
    ylim = axargs.pop('ylim', None)
    # set title and subtitle
    subtitle = axargs.pop('subtitle', None)
    # format axes
    for key in axargs:
        getattr(ax, 'set_%s' % key)(axargs[key])
    # format subtitle first
    if subtitle:
        pos = list(ax.title.get_position())
        pos[1] += 0.05
        ax.title.set_position(pos)
        ax.text(.5, 1., subtitle, transform=ax.transAxes, va='bottom',
                ha='center')
    # set minor grid for log scale
    if ax.get_xscale() == 'log':
        ax.grid(True, axis='x', which='both')
    if ax.get_yscale() == 'log':
        ax.grid(True, axis='y', which='both')
    # set limits after everything else (matplotlib might undo it)
    if xlim is not None:
        ax.set_xlim(*xlim)
    if ylim is not None:
        ax.set_ylim(*ylim)
    # save and close
    plot.save(outfile, bbox_inches=bbox_inches)
    if close:
        plot.close()


def significance_drop(outfile, old, new, show_channel_names=None, **kwargs):
    """Plot the signifiance drop for each channel
    """
    channels = sorted(old.keys())
    if show_channel_names is None:
        show_channel_names = len(channels) <= 50

    plot = Plot(figsize=(20, 5))
    plot.subplots_adjust(left=.07, right=.93)
    ax = plot.gca()
    if show_channel_names:
        plot.subplots_adjust(bottom=.4)

    winner = sorted(old.items(), key=lambda x: x[1])[-1][0]

    for i, c in enumerate(channels):
        if c == winner:
            color = 'orange'
        elif old[c] > new[c]:
            color = 'dodgerblue'
        else:
            color = 'red'
        ax.plot([i, i], [old[c], new[c]], color=color, linestyle='-',
                marker='o', markeredgecolor='k', markeredgewidth=.5,
                markersize=10, label=c, zorder=old[c])

    ax.set_xlim(-1, len(channels))
    ax.set_ybound(lower=0)

    # set xticks to show channel names
    if show_channel_names:
        ax.set_xticks(range(len(channels)))
        ax.set_xticklabels([texify(c) for c in channels])
        for i, t in enumerate(ax.get_xticklabels()):
            t.set_rotation(270)
            t.set_verticalalignment('top')
            t.set_horizontalalignment('center')
            t.set_fontsize(8)
    # or just show systems of channels
    else:
        plot.canvas.draw()
        systems = {}
        for i, c in enumerate(channels):
            sys = c.split(':', 1)[1].split('-')[0].split('_')[0]
            try:
                systems[sys][1] += 1
            except KeyError:
                systems[sys] = [i, 1]
        systems = sorted(systems.items(), key=lambda x: x[1][0])
        labels, counts = zip(*systems)
        xticks, xmticks = zip(*[(a, a+b/2.) for (a, b) in counts])
        # show ticks at the edge of each group
        ax.set_xticks(xticks, minor=False)
        ax.set_xticklabels([], minor=False)
        # show label in the centre of each group
        ax.set_xticks(xmticks, minor=True)
        for t in ax.set_xticklabels(labels, minor=True):
            t.set_rotation(270)

    kwargs.setdefault('ylabel', 'Significance')

    # create interactivity
    if outfile.endswith('.svg'):
        _finalize_plot(plot, ax, outfile.replace('.svg', '.png'),
                       close=False, **kwargs)
        tooltips = []
        ylim = ax.get_ylim()
        yoffset = (ylim[1] - ylim[0]) * 0.061
        bbox = {'fc': 'w', 'ec': '.5', 'alpha': .9, 'boxstyle': 'round'}
        xthresh = len(channels) / 10.
        for i, l in enumerate(ax.lines):
            x = l.get_xdata()[1]
            if x < xthresh:
                ha = 'left'
            elif x > (len(channels) - xthresh):
                ha = 'right'
            else:
                ha = 'center'
            y = l.get_ydata()[0] + yoffset
            c = l.get_label()
            tooltips.append(ax.annotate(texify(c), (x, y), ha=ha,
                                        zorder=ylim[1], bbox=bbox))
            l.set_gid('line-%d' % i)
            tooltips[-1].set_gid('tooltip-%d' % i)

        f = BytesIO()
        plot.savefig(f, format='svg')
        tree, xmlid = etree.XMLID(f.getvalue())
        tree.set('onload', 'init(evt)')
        for i in range(len(tooltips)):
            try:
                e = xmlid['tooltip-%d' % i]
            except KeyError:
                warnings.warn("Failed to recover tooltip %d" % i)
                continue
            e.set('visibility', 'hidden')
        for i, l in enumerate(ax.lines):
            e = xmlid['line-%d' % i]
            e.set('onmouseover', 'ShowTooltip(this)')
            e.set('onmouseout', 'HideTooltip(this)')
        tree.insert(0, etree.XML(SHOW_HIDE_JAVASCRIPT))
        etree.ElementTree(tree).write(outfile)
        plot.close()
    else:
        _finalize_plot(plot, ax, outfile, **kwargs)


def hveto_roc(outfile, rounds, figsize=[9, 6], constants=[1, 5, 10, 20],
              **kwargs):
    efficiency = []
    deadtime = []
    for r in rounds:
        try:
            efficiency.append(r.cum_efficiency[0] / r.cum_efficiency[1])
        except ZeroDivisionError:
            efficiency.append(0.)
        try:
            deadtime.append(r.cum_deadtime[0] / r.cum_deadtime[1])
        except ZeroDivisionError:
            deadtime.append(0.)
    plot = Plot(figsize=figsize)
    ax = plot.gca()
    ax.plot(deadtime, efficiency, marker='o', linestyle='-')
    try:
        xbound = 10 ** floor(log10(deadtime[0]))
    except ValueError:
        xbound = 1e-4
    try:
        ybound = 10 ** floor(log10(efficiency[0]))
    except ValueError:
        ybound = 1e-4
    bound = min(xbound, ybound)
    axargs = {
        'xlabel': 'Fractional deadtime',
        'ylabel': 'Fractional efficiency',
        'xscale': 'log',
        'yscale': 'log',
        'xlim': (bound, 1.),
        'ylim': (bound, 1.),
    }
    axargs.update(kwargs)
    # draw some eff/dt contours
    if len(constants):
        for i, c in enumerate(constants):
            g = 1 - ((i+1)/len(constants) * .5)
            x = axargs['xlim']
            y = [a * c for a in x]
            ax.plot(x, y, linestyle='--', color=(g, g, g), label=str(c))
        ax.legend(title='Eff/dt:', borderaxespad=0, bbox_to_anchor=(1.01, 1),
                  handlelength=1, handletextpad=.5, loc='upper left')
    # save and close
    _finalize_plot(plot, ax, outfile, **axargs)



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for `hveto.segments`
"""

import os
import shutil
from tempfile import NamedTemporaryFile
from unittest import mock

import pytest

from gwpy.segments import (Segment, SegmentList,
                           DataQualityFlag, DataQualityDict)

from .. import segments

TEST_SEGMENTS = SegmentList([
    Segment(0.1, 1.234567),
    Segment(5.64321, 6.234567890),
])
TEST_SEGMENTS_2 = SegmentList([Segment(round(a, 6), round(b, 6)) for
                               a, b in TEST_SEGMENTS])

TEST_FLAG = DataQualityFlag(
    known=SegmentList([Segment(0, 7)]),
    active=TEST_SEGMENTS,
    name='X1:TEST-FLAG')
TEST_DICT = DataQualityDict({
    TEST_FLAG.name: TEST_FLAG})


# -- unit tests ---------------------------------------------------------------

@mock.patch('gwpy.segments.DataQualityFlag.query', return_value=TEST_FLAG)
def test_query(dqflag):
    flag = segments.query('X1:TEST-FLAG', 0, 7)
    assert flag.known == TEST_FLAG.known
    assert flag.active == SegmentList([
        Segment((int(seg[0]), int(seg[1]))) for seg in TEST_FLAG.active])


@pytest.mark.parametrize('ncol', (2, 4))
def test_write_segments_ascii(ncol):
    with NamedTemporaryFile(suffix='.txt', delete=False) as tmp:
        segments.write_ascii(tmp.name, TEST_SEGMENTS, ncol=ncol)
        tmp.delete = True
        a = SegmentList.read(tmp.name, gpstype=float, strict=False)
        assert a == TEST_SEGMENTS_2


def test_write_segments_ascii_failure():
    with pytest.raises(ValueError) as exc:
        segments.write_ascii('test.txt', TEST_SEGMENTS, ncol=42)
    assert str(exc.value).startswith('Invalid number of columns')


@mock.patch('gwpy.segments.DataQualityDict.from_veto_definer_file')
def test_read_veto_definer_file(dqflag, tmpdir):
    dqflag.return_value = TEST_DICT
    os.chdir(str(tmpdir))
    testfile = 'https://www.w3.org/TR/PNG/iso_8859-1.txt'
    dqdict = segments.read_veto_definer_file(testfile)
    assert dqdict == TEST_DICT
    shutil.rmtree(str(tmpdir), ignore_errors=True)



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for :mod:`hveto.core`
"""

import pytest

from .. import core


def test_create_round():
    """Test creation of a `hveto.core.HvetoRound` object
    """
    r = core.HvetoRound(1, 'X1:PRIMARY')
    assert r.n == 1
    assert r.primary == 'X1:PRIMARY'


@pytest.mark.parametrize('n, mu, sig', [
    (1, 1, 0.19920008462778135),
    (100, 10, 62.26771967596927),
    (1, 100, 0.0),
])
def test_significance(n, mu, sig):
    """Test :func:`hveto.core.significance`
    """
    assert core.significance(n, mu) == pytest.approx(sig)



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for `hveto.html`
"""

import os
import shutil

import pytest

from gwdetchar.utils import parse_html

from .. import html

BANNER = """<div class="page-header" role="banner">
<h1 class="pb-2 mt-3 mb-2 border-bottom">L1 HierarchicalVeto</h1>
<h3 class="mt-3">0-100</h3>
</div>"""

NAVBAR = """<nav class="navbar fixed-top navbar-expand-md navbar-h1 shadow-sm">
<div class="container-fluid">
<div class="navbar-brand border border-white rounded">H1 Hveto</div>
<button class="navbar-toggler navbar-toggler-right" type="button" data-toggle="collapse" data-target=".navbar-collapse">
<span class="navbar-toggler-icon"></span>
</button>
<div class="collapse navbar-collapse justify-content-between">
<ul class="nav navbar-nav mr-auto">
<li class="nav-item">
<a href="#" class="nav-link">Summary</a>
</li>
<li class="nav-item dropdown">
<a href="#" class="nav-link dropdown-toggle" role="button" data-toggle="dropdown">Rounds</a>
<div class="dropdown-menu dropdown-1-col shadow">
<a href="#hveto-round-1" class="dropdown-item">1: H1:TEST-STRAIN</a>
</div>
</li>
</ul>
<ul class="nav navbar-nav">
<li class="nav-item dropdown">
<a class="nav-link dropdown-toggle" href="#" role="button" data-toggle="dropdown">Links</a>
<div class="dropdown-menu dropdown-menu-right shadow">
<h6 class="dropdown-header">Internal</h6>
<a href="about" class="dropdown-item">About this page</a>
<div class="dropdown-divider"></div>
<h6 class="dropdown-header">External</h6>
<a href="https://ldas-jobs.ligo-wa.caltech.edu/~detchar/summary/day/19800106" class="dropdown-item" target="_blank">LHO Summary Pages</a>
<a href="https://alog.ligo-wa.caltech.edu/aLOG" class="dropdown-item" target="_blank">LHO Logbook</a>
</div>
</li>
</ul>
</div>
</div>
</nav>"""  # noqa: E501


# -- unit tests ---------------------------------------------------------------

def test_banner():
    # test simple content
    out = html.banner('L1', 0, 100)
    assert parse_html(str(out)) == parse_html(BANNER)


def test_navbar():
    # test simple content
    out = html.navbar('H1', 0, ['H1:TEST-STRAIN'])
    assert parse_html(str(out)) == parse_html(NAVBAR)


@pytest.mark.parametrize('args, kwargs, result', [
    (('Key', 'Value'), {}, '<p><b>Key</b>: Value</p>'),
    (('Key', 'Value'), {'class': 'hveto', 'id_': 'test-case'},
     '<p id="test-case" class="hveto"><b>Key</b>: Value</p>'),
])
def test_bold_param(args, kwargs, result):
    h1 = parse_html(html.bold_param(*args, **kwargs))
    h2 = parse_html(result)
    assert h1 == h2


# -- end-to-end tests ---------------------------------------------------------

def test_write_hveto_page(tmpdir):
    os.chdir(str(tmpdir))
    config = 'test.ini'
    with open(config, 'w') as fobj:
        fobj.write('[test]\nchannel = X1:TEST')
    htmlv = {
        'title': 'test',
        'base': 'test',
        'config': config,
    }
    html.write_hveto_page('L1', 0, 86400, [], [], **htmlv)
    shutil.rmtree(str(tmpdir), ignore_errors=True)


def test_write_null_page(tmpdir):
    os.chdir(str(tmpdir))
    html.write_null_page('L1', 0, 86400, 'test', 'info')
    shutil.rmtree(str(tmpdir), ignore_errors=True)



# -*- coding: utf-8 -*-
# Copyright (C) Siddharth Soni (2020-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Constants for `hveto.utils`
"""

import os
from gwpy.time import from_gps

__author__ = 'Siddharth Soni <siddharth.soni@ligo.org>'


def get_hvetopath(gpstime):
    """ Returns the path of hveto output files

    Given a gpstime, the path of the folder
    containing hveto trigger files is returned

    Parameters
    ----------
    gpstime : `str` or `float`
        start time of the day for this analysis

    Returns
    _______
    path : `str`
        path to the hveto output file on the local filesystem

    Example
    _______
    >>> from hveto.const import get_hvetopath
    >>> get_hvetopath(1257811218)
    '/home/detchar/public_html/hveto/day/20191115/latest'
    """
    date = from_gps(gpstime).strftime('%Y%m%d')
    # the following hveto_dir path exists for normal Detchar workflow
    # at LHO and LLO computing clusters
    hveto_dir = '/home/detchar/public_html/hveto/day/'
    path = os.path.join(hveto_dir, date, 'latest')
    return path



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for `hveto.triggers`
"""

import pytest

from astropy.table import Table

from gwpy.segments import (Segment, SegmentList)

from .. import triggers

AUX_FILES = {
    'L1:GDS-CALIB_STRAIN': 'L1-GDS_CALIB_STRAIN_OMICRON-12345-67890.xml.gz',
    'H1:SUS-BS_M1_MASTER_OUT_F2_DQ_0_DAC':
        'H1-SUS_BS_M1_MASTER_OUT_F2_DQ_0_DAC-1126252143-22179.xml.gz',
}


def test_aux_channels_from_cache():
    cache = list(AUX_FILES.values())
    channels = triggers.find_auxiliary_channels(
        'omicron', None, None, cache=cache)
    assert channels == sorted(AUX_FILES.keys())

    channels = triggers.find_auxiliary_channels(
        'omicron', None, None, cache=cache)
    assert channels == sorted(AUX_FILES.keys())


def test_get_triggers():
    # test that trigfind raises a warning if the channel-level directory
    # doesn't exist
    with pytest.warns(UserWarning):
        out = triggers.get_triggers('X1:DOES_NOT_EXIST', 'omicron',
                                    SegmentList([Segment(0, 100)]))
    # check output type and columns
    assert isinstance(out, Table)
    for col in ['time', 'frequency', 'snr']:
        assert col in out.dtype.names



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Segment utilities for hveto
"""

from __future__ import print_function

import os
import os.path
from functools import wraps

try:
    from urllib.parse import urlparse
    from urllib.request import urlopen
except ImportError:  # python < 3
    from urlparse import urlparse
    from urllib2 import urlopen

from gwpy.segments import (DataQualityFlag, DataQualityDict)

__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
__credits__ = 'Joshua Smith <joshua.smith@ligo.org>'

DEFAULT_SEGMENT_SERVER = os.getenv('DEFAULT_SEGMENT_SERVER',
                                   'https://segments.ligo.org')


def integer_segments(f):
    @wraps(f)
    def decorated_method(*args, **kwargs):
        flag = f(*args, **kwargs)
        segs = flag.active
        flag.active = type(segs)(type(s)(int(s[0]), int(s[1])) for s in segs)
        return flag
    return decorated_method


@integer_segments
def query(flag, start, end, url=DEFAULT_SEGMENT_SERVER):
    """Query a segment database for active segments associated with a flag
    """
    return DataQualityFlag.query(flag, start, end, url=url)


def write_ascii(outfile, segmentlist, ncol=4):
    if ncol not in [2, 4]:
        raise ValueError("Invalid number of columns: %r" % ncol)
    with open(outfile, 'w') as f:
        for i, seg in enumerate(segmentlist):
            if ncol == 2:
                print("%f %f" % seg, file=f)
            else:
                print("%d\t%f\t%f\t%f" % (i, seg[0], seg[1], abs(seg)), file=f)


def read_veto_definer_file(vetofile, start=None, end=None, ifo=None):
    """Read a veto definer file, downloading it if necessary
    """
    if urlparse(vetofile).netloc:
        tmp = urlopen(vetofile)
        vetofile = os.path.abspath(os.path.basename(vetofile))
        with open(vetofile, 'w') as f:
            f.write(str(tmp.read()))
    return DataQualityDict.from_veto_definer_file(
        vetofile, format='ligolw', start=start, end=end, ifo=ifo)



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""Check if a trigger was vetoed by a specified hveto run
"""

import os
import sys
import json

from gwpy.segments import SegmentList

from gwdetchar import cli

from .. import __version__

__author__ = 'Joshua Smith <joshua.smith@ligo.org>'


# -- parse command line -------------------------------------------------------

def _abs_path(p):
    return os.path.abspath(os.path.expanduser(p))


def create_parser():
    """Create a command-line parser for this entry point
    """
    parser = cli.create_parser(
        description=__doc__,
        version=__version__,
    )
    parser.add_argument(
        '-t',
        '--trigger-time',
        required=True,
        help='GPS time of the trigger',
    )
    parser.add_argument(
        '-d',
        '--directory',
        required=True,
        type=_abs_path,
        help=('path to hveto-generated folder containing '
              'a summary-stats.json file'),
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_const',
        dest='loglevel',
        const='DEBUG',
        default='INFO',
        help='Log verbose output',
    )
    return parser


# -- main code block ----------------------------------------------------------

def main(args=None):
    """Run the trace tool
    """
    parser = create_parser()
    args = parser.parse_args(args=args)
    directory = args.directory

    logger = cli.logger(name='hveto.trace', level=args.loglevel)
    logger.debug('Running in verbose mode')
    logger.debug('Search directory: %s' % directory)

    trigger_time = float(args.trigger_time)
    if directory[-1] != '/':
        directory += '/'

    try:
        segment_stats = json.load(open('%ssummary-stats.json' % directory))
    except IOError:
        logger.error("'summary-stats.json' was not found "
                     "in the input directory")
        sys.exit(0)

    for i, cround in enumerate(segment_stats['rounds']):
        seg_files = filter(
            lambda f_name: '.txt' in f_name, cround[u'files'][u'VETO_SEGS'])
        for f in seg_files:
            segments = SegmentList.read(os.path.join(directory, f))
            for segment in segments:
                if segment[0] <= trigger_time <= segment[1]:
                    logger.info('Signal was vetoed in round %d by '
                                'segment %s' % ((i + 1), segment))
                    logger.debug('Winner: %s' % cround['name'])
                    logger.debug('Significance: %s' % cround['significance'])
                    logger.debug('SNR: %s' % cround['snr'])
                    logger.debug('Window: %s' % cround['window'])
                    sys.exit(0)

    logger.info('Signal was not vetoed.')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright (C) Joshua Smith (2016-)
#
# This file is part of the hveto python package.
#
# hveto is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# hveto is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hveto.  If not, see <http://www.gnu.org/licenses/>.

"""HTML utilities for hveto
"""

import os.path
from functools import wraps

from MarkupPy import markup

from gwdetchar.io import html as gwhtml

from ._version import get_versions

__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
__credits__ = 'Josh Smith, Joe Areeda, Alex Urban'


# -- HTML construction --------------------------------------------------------

def navbar(ifo, gpstime, winners=[]):
    """Initialise a new `markup.page`

    Parameters
    ----------
    ifo : `str`
        the interferometer prefix

    gpstime : `float`
        the central GPS time of the analysis

    winners : `list`
        list of round winners for navbar table of contents

    Returns
    -------
    page : `markup.page`
        the structured markup to open an HTML document
    """
    (brand, class_) = gwhtml.get_brand(ifo, 'Hveto', gpstime, about='about')
    # channel navigation
    links = [['Summary', '#']]
    if not winners:
        links.append(['Rounds', '#rounds'])
    else:
        winners = [['%d: %s' % (i + 1, channel), '#hveto-round-%s' % (i + 1)]
                   for i, channel in enumerate(winners)]
        links.append(['Rounds', winners])
    return gwhtml.navbar(links, brand=brand, class_=class_)


def banner(ifo, start, end):
    """Initialise a new markup banner

    Parameters
    ----------
    ifo : `str`
        the interferometer prefix
    start : `int`
        the GPS start time of the analysis
    end : `int`
        the GPS end time of the analysis

    Returns
    -------
    page : `markup.page`
        the structured markup to open an HTML document
    """
    # create page
    page = markup.page()
    # write banner
    page.div(class_='page-header', role='banner')
    page.h1("%s HierarchicalVeto" % ifo, class_='pb-2 mt-3 mb-2 border-bottom')
    page.h3("%d-%d" % (start, end), class_='mt-3')
    page.div.close()
    return page()


def wrap_html(func):
    """Decorator to wrap a function with `init_page` and `close_page` calls

    This allows inner HTML methods to be written with minimal arguments
    and content, hopefully making things simpler
    """
    @wraps(func)
    def decorated_func(ifo, start, end, *args, **kwargs):
        # set page init args
        initargs = {
            'title': '%s Hveto | %d-%d' % (ifo, start, end),
            'base': os.path.curdir,
        }
        for key in ['title', 'base']:
            if key in kwargs:
                initargs[key] = kwargs.pop(key)
        # find outdir
        outdir = kwargs.pop('outdir', initargs['base'])
        winners = kwargs.pop('winners', [])
        if not os.path.isdir(outdir):
            os.makedirs(outdir)
        # write about page
        try:
            config = kwargs.pop('config')
        except KeyError:
            about = None
        else:
            iargs = initargs.copy()
            aboutdir = os.path.join(outdir, 'about')
            if iargs['base'] == os.path.curdir:
                iargs['base'] = os.path.pardir
            about = write_about_page(ifo, start, end, config, outdir=aboutdir,
                                     **iargs)
            if os.path.basename(about) == 'index.html':
                about = about[:-10]
        # open page
        nav = navbar(ifo, start, winners)
        page = gwhtml.new_bootstrap_page(navbar=nav, **initargs)
        page.add(banner(ifo, start, end))
        # write content
        page.add(str(func(*args, **kwargs)))
        # close page with custom footer
        index = os.path.join(outdir, 'index.html')
        version = get_versions()['version']
        commit = get_versions()['full-revisionid']
        url = 'https://github.com/gwdetchar/hveto/tree/{}'.format(commit)
        gwhtml.close_page(page, index, about=about,
                          link=('hveto-%s' % version, url, 'GitHub'),
                          issues='https://github.com/gwdetchar/hveto/issues')
        return index
    return decorated_func


# -- Utilities ----------------------------------------------------------------

def bold_param(key, value, **attrs):
    """Write a (key, value) pair in HTML as ``<p><b>key</b>: value</p>``

    Parameters
    ----------
    key : `str`
        the element to be rendered in bold
    value : `str`
        the explanation/description of the key
    **attrs
        HTML attributes for the containing ``<p>`` tag

    Returns
    -------
    html : `str`
    """
    return markup.oneliner.p('<b>%s</b>: %s' % (key, value), **attrs)


# -- Hveto HTML ---------------------------------------------------------------

def write_summary(
        rounds, plots=[], header='Summary', plotsperrow=4,
        tableclass='table table-sm table-hover'):
    """Write the Hveto analysis summary HTML

    Parameters
    ----------
    rounds : `list` of `HvetoRound`
        the `list` of round objects produced by this analysis
    plots : `list` of `str`, optional
        the `list` of summary plots to display underneath the summary table
    header : `str`, optional
        the text for the section header (``<h2``>)
    plotsperrow : `int`, optional
        the number of plots to display in each row
    tableclass : `str`, optional
        the ``class`` for the summary ``<table>``

    Returns
    -------
    page : `~MarkupPy.markup.page`
        the formatted markup object containing the analysis summary table,
        and images
    """
    page = markup.page()
    page.h2(header, class_='mt-4')
    page.table(class_=tableclass)
    page.caption("Summary of this HierarchichalVeto analysis.")
    # make header
    page.thead()
    page.tr()
    for header in ['Round', 'Winner', 'Twin [s]', 'SNR Thresh', 'Significance',
                   'Use [%]', 'Efficiency [%]', 'Deadtime [%]',
                   'Cum. efficiency [%]', 'Cum. deadtime [%]']:
        page.th(header, scope='row')
    page.tr.close()
    page.thead.close()
    # make body
    page.tbody()
    for r in rounds:
        page.tr()
        # link round down page
        page.td(gwhtml.html_link(
            '#hveto-round-%d' % r.n, r.n, target=None,
            title="Jump to round %d details" % r.n,
            style='color: inherit;',
        ))
        # link name to CIS
        page.td(gwhtml.cis_link(r.winner.name))
        for attr in ['window', 'snr', 'significance']:
            v = getattr(r.winner, attr)
            if isinstance(v, float):
                page.td('%.2f' % v)
            else:
                page.td(str(v))
        for attr in ['use_percentage', 'efficiency', 'deadtime',
                     'cum_efficiency', 'cum_deadtime']:
            a, b = getattr(r, attr)
            try:
                pc = a/b * 100.
            except ZeroDivisionError:
                pc = 0.
            if attr.endswith('deadtime'):
                page.td('%.2f<br><small>[%.2f/%.2f]</small>' % (pc, a, b))
            else:
                page.td('%.2f<br><small>[%d/%d]</small>' % (pc, a, b))
        page.tr.close()
    page.tbody.close()

    page.table.close()

    # scaffold plots
    if plots:
        page.div(class_='card border-light card-body scaffold shadow-sm')
        page.add(gwhtml.scaffold_plots(plots, nperrow=plotsperrow))
        page.div.close()  # card border-light card-body scaffold shadow-sm
    return page()


def write_round(round_, context):
    """Write the HTML summary for a specific round

    Parameters
    ----------
    round_ : `HvetoRound`
        the analysis round object

    context : `str`
        context for bootstrap objects, default: info

    Returns
    -------
    page : `~MarkupPy.markup.page`
        the formatted HTML for this round
    """
    page = markup.page()
    page.div(class_='card card-%s mb-3 shadow-sm' % context)
    # -- make heading
    page.div(class_='card-header pb-0')
    # heading
    page.h5('Round %d, Winner = %s, window = %s, SNR thresh = %s'
            % (round_.n, round_.winner.name, round_.winner.window,
               round_.winner.snr),
            class_='card-title', id_='hveto-round-%d' % round_.n)
    page.div.close()  # card-header pb-0

    # -- make body
    page.div(class_='card-body')
    page.ul(class_='list-group')
    page.li(class_='list-group-item flex-column align-items-start')
    page.div(class_='row')
    # summary information
    page.div(class_='col-md-3', id_='hveto-round-%d-summary' % round_.n)
    page.add(bold_param('Winner', round_.winner.name))
    page.add(bold_param('SNR threshold', round_.winner.snr))
    page.add(bold_param('Window', round_.winner.window))
    page.add(bold_param('Significance', '%.2f' % round_.winner.significance))
    for desc, tag in zip(
            ['Veto segments', 'Veto triggers', 'Vetoed primary triggers',
             'Unvetoed primary triggers'],
            ['VETO_SEGS', 'WINNER', 'VETOED', 'RAW']):
        if isinstance(round_.files[tag], str):
            files = [round_.files[tag]]
        else:
            files = round_.files[tag]
        link = ' '.join([gwhtml.html_link(
            f, '[%s]' % os.path.splitext(f)[1].strip('.'),
            style='color: inherit;') for f in files])
        page.add(bold_param(desc, link))
    # link omega scans if generated
    if round_.scans is not None:
        page.p('<b>Omega scans:</b>')
        for t in round_.scans:
            page.p()
            page.a('%s [SNR %.1f]' % (t['time'], t[round_.rank]),
                   href='./scans/%s/' % t['time'], **{
                       'class_': 'fancybox',
                       'style': 'color: inherit;',
                       'target': '_blank'})
            for c, tag in zip([round_.primary, round_.winner.name],
                              ['Primary', 'Auxiliary']):
                caption = 'Omega scan of %s at %s' % (c, t['time'])
                png = ('./scans/%s/plots/%s-qscan_whitened-1.png'
                       % (t['time'], c.replace('-', '_').replace(':', '-')))
                page.a('[%s]' % tag[0].lower(), class_='fancybox',
                       href=png, title=caption,
                       style='color: inherit;',
                       **{'data-fancybox-group': 'omega-preview',
                          'data-fancybox': 'gallery'})
            page.p.close()
    page.div.close()  # col
    # plots
    page.div(class_='col-md-9', id_='hveto-round-%d-plots' % round_.n)
    page.add(gwhtml.scaffold_plots(round_.plots[:-1], nperrow=4))
    # add significance drop plot at end
    page.div(class_='row')
    page.div(class_='col-sm-12')
    page.add(gwhtml.fancybox_img(round_.plots[-1], lazy=True))
    page.div.close()  # col-sm-12
    page.div.close()  # row
    page.div.close()  # col-md-8

    page.div.close()  # row
    page.li.close()  # list-group-item flex-column align-items-start
    page.ul.close()  # list-group
    # close and return
    page.div.close()  # card-body
    page.div.close()  # card
    return page()


# reminder: wrap_html automatically prepends the (ifo, start, end) args,
# and at least the outdir kwarg, so you should include those in the docstring,
# but not in the actual function declaration - the decorator will take care of
# that for you.

@wrap_html
def write_hveto_page(rounds, plots, context='default'):
    """Write the Hveto results to HTML

    Parameters
    ----------
    ifo : `str`
        the prefix of the interferometer used in this analysis
    start  : `int`
        the GPS start time of the analysis
    end : `int`
        the GPS end time of the analysis
    rounds : `list` of `HvetoRound`
        the rounds produced by this analysis
    plots : `list` of `str`
        the `list` of summary plots
    outdir : `str`, optional
        the output directory for the HTML
    winners : `list` of `str`, optional
        list of channels that won each round
    context : `str`, optional
        the bootstrap context class for this result, see the bootstrap
        docs for more details

    Returns
    -------
    index : `str`
        the path of the HTML written for this analysis
    """
    page = markup.page()
    page.add(write_summary(rounds, plots))
    page.h2('Round details', class_='mt-4', id_='rounds')
    for r in rounds:
        page.add(write_round(r, context=context))
    return page


@wrap_html
def write_null_page(reason, context='info'):
    """Write the Hveto results to HTML

    Parameters
    ----------
    ifo : `str`
        the prefix of the interferometer used in this analysis
    start  : `int`
        the GPS start time of the analysis
    end : `int`
        the GPS end time of the analysis
    reason : `str`
        the explanation for this null result
    context : `str`, optional
        the bootstrap context class for this result, see the bootstrap
        docs for more details
    outdir : `str`, optional
        the output directory for the HTML

    Returns
    -------
    index : `str`
        the path of the HTML written for this analysis
    """
    return gwhtml.alert(reason, context=context, dismiss=False)


@wrap_html
def write_about_page(configfile):
    """Write a page explaining how an hveto analysis was completed

    Parameters
    ----------
    ifo : `str`
        the prefix of the interferometer used in this analysis
    start  : `int`
        the GPS start time of the analysis
    end : `int`
        the GPS end time of the analysis
    configfile : `str`
        the path of the configuration file to embed
    outdir : `str`, optional
        the output directory for the HTML

    Returns
    -------
    index : `str`
        the path of the HTML written for this analysis
    """
    return gwhtml.about_this_page(configfile)
